package jcg.zheng.demo;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Set;

public class IteratorExample {

	private static void addFiveElements(Collection<String> list) {
		list.add("One");
		list.add("Two");
		list.add("Three");
		list.add("Four");
		list.add("Five");
	}

	private static void demoIterator_listIterator(List<String> list) {
		addFiveElements(list);

		iteratorWithforEachRemaining(list);
		iteratorWithWhileLoop(list);
		iteratorWithForLoop(list);
		listIteratorForwardsTraverse(list);
		listIteratorBackwardsTraverse(list);
		listIterator_replace_element(list);
		listIterator_add_remove(list);
	}

	private static void iterator_set(Set<String> items) {

		addFiveElements(items);
		Iterator<String> it = items.iterator();
		System.out.println("Traversing set via Iterator");
		while (it.hasNext()) {
			System.out.println(it.next());
		}

	}

	private static void iteratorWithforEachRemaining(List<String> list) {
		System.out.println("Iterator using forEachRemaining: ");
		Iterator<String> it1 = list.iterator();

		it1.forEachRemaining(System.out::println);
	}

	private static void iteratorWithForLoop(List<String> list) {
		System.out.println("Iterator using for loop: ");
		for (Iterator<String> it2 = list.iterator(); it2.hasNext();) {
			System.out.println("Next element: " + it2.next());
		}
	}

	private static void iteratorWithWhileLoop(List<String> list) {
		System.out.println("Iterator using while loop: ");
		Iterator<String> it1 = list.iterator();
		while (it1.hasNext()) {
			System.out.println("Next element: " + it1.next());
		}

	}

	private static void listIterator_replace_element(List<String> list) {
		System.out.println("Add|Replace element: ");
		for (ListIterator<String> listIt = list.listIterator(); listIt.hasNext();) {
			String checkItem = listIt.next();
			if ("Four".equalsIgnoreCase(checkItem)) {
				listIt.set("Replaced Four"); // replace the element the iterator is currently at
			}

		}

		System.out.println("Traverse with forEach: ");
		list.forEach(System.out::println);
	}

	private static void listIterator_add_remove(List<String> list) {
		ListIterator<String> lit = list.listIterator(0);

		lit.next();
		lit.add("One more element");
		System.out.println("Modified list after the insertion of the new element");

		System.out.println("Index of next element: " + lit.nextIndex());
		System.out.println("Index of previous element: " + lit.previousIndex());

		for (lit = list.listIterator(); lit.hasNext();) {
			System.out.println("Next element: " + lit.next());
		}

		lit.previous();
		lit.remove();

		System.out.println("Modified list after the removal of an element");
		for (lit = list.listIterator(); lit.hasNext();) {
			System.out.println("Next element: " + lit.next());
		}
	}

	private static void listIteratorBackwardsTraverse(List<String> list) {
		ListIterator<String> lit;
		System.out.println("List iterator (backward iteration): ");
		lit = list.listIterator(list.size());
		while (lit.hasPrevious()) {
			System.out.println("Previous element: " + lit.previous());

		}

	}

	private static void listIteratorForwardsTraverse(List<String> list) {
		System.out.println("List iterator (forward iteration): ");
		ListIterator<String> lit = list.listIterator();
		while (lit.hasNext()) {
			System.out.println("Next element: " + lit.next());

		}
	}

	public static void main(String args[]) {
		System.out.println("\nDemo with ArrayList\n");
		demoIterator_listIterator(new ArrayList<String>());
		System.out.println("\nDemo with LinkedList\n");
		demoIterator_listIterator(new LinkedList<String>());
		System.out.println("\nDemo with Set\n");
		iterator_set(new HashSet<String>());

	}
}
